import re
from melusine.config.config import ConfigJsonReader

conf_reader = ConfigJsonReader()
config = conf_reader.get_config_file()
regex_transfer_header = config['regex']['manage_transfer_reply']['transfer_header']
regex_answer_header = config['regex']['manage_transfer_reply']['answer_header']
regex_begin_transfer = config['regex']['manage_transfer_reply']['begin_transfer']
regex_begin_transfer_cons = config['regex']['manage_transfer_reply']['begin_transfer_cons']
regex_extract_from = config['regex']['manage_transfer_reply']['extract_from']
regex_extract_to = config['regex']['manage_transfer_reply']['extract_to']
regex_extract_date = config['regex']['manage_transfer_reply']['extract_date']
regex_extract_header = config['regex']['manage_transfer_reply']['extract_header']


def add_boolean_transfer(row):
    """Compute boolean Series which return True if the "header" starts with given
    regex 'answer_subject', False if not.

    To be used with methods such as: `apply(func, axis=1)` or
    `apply_by_multiprocessing(func, axis=1, **kwargs)`.

    Parameters
    ----------
    row : row of pd.Dataframe, columns ['header']

    Returns
    -------
    pd.Series

    Examples
    --------
        >>> import pandas as pd
        >>> data = pd.read_pickle('./tutorial/data/emails_anonymized.pickle')
        >>> # data contains a 'header' column

        >>> from melusine.prepare_email.manage_transfer_reply import add_boolean_transfer
        >>> add_boolean_transfer(data.iloc[0])  # apply for 1 sample
        >>> data.apply(add_boolean_transfer, axis=1)  # apply to all samples

    """
    is_transfer = False
    try:
        if re.match(regex_transfer_header, row['header']):
            is_transfer = True
    except Exception as e:
        pass

    return is_transfer


def add_boolean_answer(row):
    """Compute boolean Series which return True if the "header" starts with given
    regex 'transfer_subject', False if not.

    To be used with methods such as: `apply(func, axis=1)` or
    `apply_by_multiprocessing(func, axis=1, **kwargs)`.

    Parameters
    ----------
    row : row of pd.Dataframe, columns ['header']

    Returns
    -------
    pd.Series

    Examples
    --------
        >>> import pandas as pd
        >>> data = pd.read_pickle('./tutorial/data/emails_anonymized.pickle')
        >>> # data contains a 'header' column

        >>> from melusine.prepare_email.manage_transfer_reply import add_boolean_answer
        >>> add_boolean_answer(data.iloc[0])  # apply for 1 sample
        >>> data.apply(add_boolean_answer, axis=1)  # apply to all samples

    """
    is_answer = False
    try:
        if re.match(regex_answer_header, row['header']):
            is_answer = True
    except Exception as e:
        pass

    return is_answer


def check_mail_begin_by_transfer(row):
    """Compute boolean Series which return True if the "body" starts with given
    regex 'begin_transfer', False if not.

    To be used with methods such as: `apply(func, axis=1)` or
    `apply_by_multiprocessing(func, axis=1, **kwargs)`.

    Parameters
    ----------
    row : row of pd.Dataframe, columns ['body']

    Returns
    -------
    pd.Series

    Examples
    --------
        >>> import pandas as pd
        >>> data = pd.read_pickle('./tutorial/data/emails_anonymized.pickle')
        >>> # data contains a 'body' column

        >>> from melusine.prepare_email.manage_transfer_reply import check_mail_begin_by_transfer
        >>> check_mail_begin_by_transfer(data.iloc[0])  # apply for 1 sample
        >>> data.apply(check_mail_begin_by_transfer, axis=1)  # apply to all samples

    """
    is_begin_by_transfer = False
    try:
        if re.search(regex_begin_transfer, row['body']):
            is_begin_by_transfer = True
        if re.search(regex_begin_transfer_cons, row['body']):
            is_begin_by_transfer = True
    except Exception as e:
        pass

    return is_begin_by_transfer


def update_info_for_transfer_mail(row):
    """Extracts and updates informations from forwarded mails, such as: body,
    from, to, header, date.
    - It changes the header by the initial subject (extracted from forward
    email).
    - It removes the header from emails' body.

    To be used with methods such as: `apply(func, axis=1)` or
    `apply_by_multiprocessing(func, axis=1, **kwargs)`.

    Parameters
    ----------
    row : row of pd.Dataframe,
    columns ['body', 'header', 'from', 'to', 'date', 'is_begin_by_transfer']

    Returns
    -------
    pd.DataFrame

    Examples
    --------
        >>> import pandas as pd
        >>> from melusine.prepare_email.manage_transfer_reply import check_mail_begin_by_transfer
        >>> data = pd.read_pickle('./tutorial/data/emails_anonymized.pickle')
        >>> data['is_begin_by_transfer'] = data.apply(check_mail_begin_by_transfer, axis=1)
        >>> # data contains columns ['from', 'to', 'date', 'header', 'body', 'is_begin_by_transfer']

        >>> from melusine.prepare_email.manage_transfer_reply import update_info_for_transfer_mail
        >>> update_info_for_transfer_mail(data.iloc[0])  # apply for 1 sample
        >>> data.apply(update_info_for_transfer_mail, axis=1)  # apply to all samples

    """
    try:
        if row['is_begin_by_transfer']:
            row['from'] = re.split(regex_extract_from, row['body'])[1]
            row['to'] = re.split(regex_extract_to, row['body'])[1]
            row['date'] = re.split(regex_extract_date, row['body'])[1]
            row['header'] = re.split(regex_extract_header, row['body'])[1]
            row['body'] = ''.join(row['body'].split(
                re.findall(regex_extract_header, row['body'])[0])[1:])

    except Exception as E:
        pass

    return row



import re
import pandas as pd
from collections import Counter
from sklearn import preprocessing
from sklearn.base import BaseEstimator, TransformerMixin


class MetaExtension(BaseEstimator, TransformerMixin):
    """Transformer which creates 'extension' feature extracted
    from regex in metadata. It extracts extension of mail adresses.

    Compatible with scikit-learn API.
    """

    def __init__(self):
        self.le_extension = preprocessing.LabelEncoder()

    def fit(self, X, y=None):
        """ Fit LabelEncoder on encoded extensions."""
        X['extension'] = X['from'].apply(self.get_extension)
        self.top_extension = self.get_top_extension(X, n=100)
        X['extension'] = X['extension'].apply(
            lambda x: self.encode_extension(x, self.top_extension))
        self.le_extension.fit(X['extension'])
        return self

    def transform(self, X):
        """Encode extensions"""
        X['extension'] = X['from'].apply(self.get_extension)
        X['extension'] = X['extension'].apply(
            lambda x: self.encode_extension(x, self.top_extension))
        X['extension'] = self.le_extension.transform(X['extension'])
        return X

    @staticmethod
    def get_extension(x):
        """Gets extension from email address."""
        try:
            extension = re.findall(r'\@([^.]+)', x)[0]
        except Exception as e:
            return ''
        return extension

    @staticmethod
    def get_top_extension(X, n=100):
        "Returns list of most common extensions."
        a = Counter(X['extension'].values)
        a = a.most_common(n)
        a = [x[0] for x in a]
        return a

    @staticmethod
    def encode_extension(x, top_ext):
        """Encode most common extensions and set the rest to 'other'."""
        if x in top_ext:
            return x
        else:
            return 'other'


class MetaDate(BaseEstimator, TransformerMixin):
    """Transformer which creates new features from dates such as:
        - hour
        - minute
        - dayofweek

    Compatible with scikit-learn API.

    Parameters
    ----------
    date_format : str, optional
        Regex to extract date from text.

    date_format : str, optional
        A date format.
    """

    def __init__(self,
                 regex_date_format=r'\w+ (\d+) (\w+) (\d{4}) (\d{2}) h (\d{2})',
                 date_format='%d/%m/%Y %H:%M'):
        self.regex_date_format = regex_date_format
        self.date_format = date_format
        self.month = {
            'janvier': '1',
            'février': '2',
            'mars': '3',
            'avril': '4',
            'mai': '5',
            'juin': '6',
            'juillet': '7',
            'août': '8',
            'septembre': '9',
            'octobre': '10',
            'novembre': '11',
            'décembre': '12',
        }

    def fit(self, X, y=None):
        """Unused method. Defined only for compatibility with scikit-learn API.
        """
        return self

    def transform(self, X):
        """Transform date to hour, min, day features."""
        X['date'] = X['date'].apply(self.date_formatting,
                                    args=(self.regex_date_format, ))
        X['date'] = pd.to_datetime(X['date'],
                                   format=self.date_format,
                                   infer_datetime_format=False,
                                   errors='coerce')
        X['hour'] = X['date'].apply(self.get_hour)
        X['min'] = X['date'].apply(self.get_min)
        X['dayofweek'] = X['date'].apply(self.get_dayofweek)
        return X

    def date_formatting(self, x, regex_format):
        """Set a date in the right format"""
        try:
            e = re.findall(regex_format, x)[0]
            date = e[0]+'/'+e[1]+'/'+e[2]+' '+e[3]+':'+e[4]
            for m, m_n in self.month.items():
                date = date.replace(m, m_n)
        except Exception as e:
            return x
        return date

    @staticmethod
    def get_hour(x):
        """Get hour from date"""
        try:
            return x.hour
        except Exception as e:
            return 0

    @staticmethod
    def get_min(x):
        """Get minutes from date"""
        try:
            return x.minute
        except Exception as e:
            return 0

    @staticmethod
    def get_dayofweek(x):
        """Get day of the week from date"""
        try:
            return x.dayofweek
        except Exception as e:
            return 0


class Dummifier(BaseEstimator, TransformerMixin):
    """Transformer to dummifies categorial features.
    Compatible with scikit-learn API.
    """

    def __init__(self,
                 columns_to_dummify=['extension', 'dayofweek', 'hour', 'min'],
                 copy=True):
        self.columns_to_dummify = columns_to_dummify
        self.copy = copy
        pass

    def fit(self, X, y=None):
        """Store dummified features to avoid inconsistance of
        new data which could contain new labels (unknown from train data).
        """
        self.X_ = pd.get_dummies(
            X, columns=self.columns_to_dummify, prefix_sep='__', dummy_na=False)

        dummies_ = tuple([col + '__' for col in self.columns_to_dummify])
        self.dummy_features = [c for c in self.X_.columns if c.startswith(dummies_)]

        return self

    def transform(self, X, y=None):
        """Dummify features and keep only common labels with pretrained data.
        """
        if self.copy:
            X_ = X.copy()
        else:
            X_ = X

        X_ = pd.get_dummies(
            X_, columns=self.columns_to_dummify, prefix_sep='__', dummy_na=False)

        return X_[self.dummy_features]



"""
Cleaning of the body and the header
"""

import unidecode, unicodedata
import re
from melusine.config import ConfigJsonReader

conf_reader = ConfigJsonReader()
config = conf_reader.get_config_file()
REGEX_CLEAN = config["regex"]['cleaning']
regex_flags_dict = REGEX_CLEAN["flags_dict"]
regex_clean_header_dict = REGEX_CLEAN["clean_header_dict"]
regex_remove_multiple_spaces_list = REGEX_CLEAN["remove_multiple_spaces_list"]


def clean_body(row, flags=True):
    """Clean body column. The cleaning involves the following operations:
        - Cleaning the text
        - Removing the multiple spaces
        - Flagging specific items (postal code, phone number, date...)

    Parameters
    ----------
    row : row of pandas.Dataframe object,
        Data contains 'last_body' column.

    flags : boolean, optional
        True if you want to flag relevant info, False if not.
        Default value, True.

    Returns
    -------
    row of pandas.DataFrame object or pandas.Series if apply to all DF.
    """
    text = str(row["last_body"])
    clean_body = clean_text(text)
    clean_body = flag_items(clean_body, flags=flags)
    return clean_body


def clean_header(row, flags=True):
    """Clean the header column. The cleaning involves the following operations:
        - Removing the transfers and answers indicators
        - Cleaning the text
        - Flagging specific items (postal code, phone number, date...)

    Parameters
    ----------
    row : row of pandas.Dataframe object,
        Data contains 'header' column.

    flags : boolean, optional
        True if you want to flag relevant info, False if not.
        Default value, True.

    Returns
    -------
    row of pd.DataFrame object or pandas.Series if apply to all DF.
    """
    text = str(row["header"])
    clean_header = remove_transfer_answer_header(text)
    clean_header = clean_text(clean_header)
    clean_header = flag_items(clean_header, flags=flags)
    return clean_header


def clean_text(text):
    """Clean a string. The cleaning involves the following operations:
        - Putting all letters to lowercase
        - Removing all the accents
        - Removing all line breaks
        - Removing all symbols and punctuations
        - Removing the multiple spaces

    Parameters
    ----------
    text : str

    Returns
    -------
    str
    """
    text = text_to_lowercase(text)
    text = remove_accents(text)
    text = remove_line_break(text)
    text = remove_superior_symbol(text)
    text = remove_apostrophe(text)
    text = remove_multiple_spaces_and_strip_text(text)
    return text


def text_to_lowercase(text):
    """Set all letters to lowercase"""
    return text.lower()


def remove_accents(text, use_unidecode=False):
    """
    Remove accents from text
    Using unidecode is more powerful but much more time consuming
    Exemple: the joined 'ae' character is converted to 'a' + 'e' by unidecode while it is suppressed by unicodedata.

    """
    if use_unidecode:
        return unidecode.unidecode(text)
    else:
        utf8_str = unicodedata.normalize('NFKD', text).encode('ASCII', 'ignore').decode("utf-8")
        return utf8_str


def remove_line_break(text):
    """Remove line breaks from text"""
    return text.replace('\n', '')


def remove_superior_symbol(text):
    """Remove superior and inferior symbols from text"""
    text = text.replace('>', '')
    text = text.replace('<', '')
    return text


def remove_apostrophe(text):
    """Remove apostrophes from text"""
    return text.replace('\'', ' ')


def remove_multiple_spaces_and_strip_text(text):
    """Remove multiple spaces, strip text, and remove '-', '*' characters.

    Parameters
    ----------
    text : str,
        Header content.

    Returns
    -------
    str

    """
    for regex_remove_multiple_spaces in regex_remove_multiple_spaces_list:
        text = re.sub(regex_remove_multiple_spaces, ' ', text)
        text = text.strip()
    return text


def flag_items(text, flags=True):
    """Flag relevant information
        ex : amount, phone number, email address, postal code (5 digits)..

    Parameters
    ----------
    text : str,
        Body content.

    flags : boolean, optional
        True if you want to flag relevant info, False if not.
        Default value, True.

    Returns
    -------
    str

    """
    if flags:
        for regex, value in regex_flags_dict.items():
            text = re.sub(pattern=regex, repl=value,
                          string=text, flags=re.IGNORECASE)
        return text
    else:
        return text


def remove_transfer_answer_header(text):
    """Remove historic and transfers indicators in the header.
    Ex: "Tr:", "Re:", "Fwd", etc.

    Parameters
    ----------
    text : str,
        Header content.

    Returns
    -------
    str

    """
    for regex, value in regex_clean_header_dict.items():
        text = re.sub(pattern=regex, repl=value,
                      string=text, flags=re.IGNORECASE)
    return text



import logging
import re
import numpy as np
import pandas as pd
from multiprocessing import Pool

from sklearn.base import BaseEstimator, TransformerMixin
from melusine.utils.multiprocessing import apply_by_multiprocessing

def aggregation_percentile_60(x):
    """
    Explicit definition of the percentile 60 function.
    This is done because lambda function can not be used in multiprocessing (can't be pickled).
    NOT OK : aggregation_function_email_wise = lambda x: np.percentile(x, 60)
    OK : aggregation_function_email_wise = aggregation_percentile_60

    Parameters
    ----------
    x: list or numpy array
        list of scores associated with the tokens in the document
    """
    return np.percentile(x, 60)

class SemanticDetector(BaseEstimator, TransformerMixin):
    """Class to fit a Lexicon on an embedding and predicts the
    Attributes
    ----------
    base_seed_words : list,
        Seedwords list containing the seedwords for computing the Lexicons given by the User.
    seed_list : list,
        Same as base_seed_words but only the seeds present in the embedding vocabulary are kept.
    n_jobs : int,
        Number of CPUs to use to rate the emails.
    progress_bar : bool,
        Whether to print or not the progress bar while rating the emails.
    extend_seed_word_list : bool,
        Whether to use the seedwords as prefixes.
    seed_dict : dict,
        Filled if extend_seed_word_list==True. Key : prefix, Value : list of seedwords having this prefix in the vocabulary
    tokens_column : str,
        Name of the column in the Pandas Dataframe on which the polarity scores will be computed. Must be a column of tokens.
    normalize_scores : bool,
        Whether or not to normalize the lexicons' scores (so that they are centered around 0 and with a variance of 1)
    lexicon_dict : dict,
        Key : Seedword, Value : dict having all the embedding vocabulary as keys, and their semantic polarity value (cosine similarity) towards the seedword as values
    normalized_lexicon_dict : dict,
        Filled only if nomalize_scores==True. Contains the same as lexicon_dict, but with the normalized polarity values instead.
    aggregation_function_seed_wise : function,
        Function to aggregate the scores returned by all the different Lexicons associated to the seedwords, for a specific token (default=np.max)
    aggregation_function_email_wise : function,
        Function to aggregate the scores given to the tokens in a e-mail (default=np.percentile(.,60))

    Examples
    --------

    """

    def __init__(self, base_seed_words, tokens_column, n_jobs=1,
                 progress_bar=False,
                 extend_seed_word_list=False,
                 normalize_lexicon_scores=False,
                 aggregation_function_seed_wise=np.max,
                 aggregation_function_email_wise=aggregation_percentile_60
                 ):
        """
        Parameters
        ----------
        base_seed_words : list,
            Seedwords list containing the seedwords for computing the Lexicons given by the User.
        tokens_column : str,
            Name of the column in the Pandas Dataframe on which the polarity scores will be computed. Must be a column of tokens.
        n_jobs : int,
            Number of CPUs to use to rate the emails.
        progress_bar : bool,
            Whether to print or not the progress bar while rating the emails.
        extend_seed_word_list : bool,
            Whether to use the seedwords as prefixes.
        normalize_scores : bool,
            Whether or not to normalize the lexicons' scores (so that they are centered around 0 and with a variance of 1)
        aggregation_function_seed_wise : function,
            Function to aggregate the scores returned by all the different Lexicons associated to the seedwords, for a specific token (default=np.max)
        aggregation_function_email_wise : function,
            Function to aggregate the scores given to the tokens in a e-mail (default=np.percentile(.,60))
        """

        self.n_jobs = n_jobs
        self.progress_bar = progress_bar

        self.base_seed_words = base_seed_words
        self.seed_dict = {word: [] for word in self.base_seed_words}
        self.seed_list = base_seed_words
        self.extend_seed_word_list = extend_seed_word_list
        self.tokens_column = tokens_column
        self.normalize_lexicon_scores = normalize_lexicon_scores

        self.lexicon_dict = {}
        self.normalized_lexicon_dict = {}

        self.aggregation_function_seed_wise = aggregation_function_seed_wise
        self.aggregation_function_email_wise = aggregation_function_email_wise

    def __getstate__(self):
        """should return a dict of attributes that will be pickled
        To override the default pickling behavior and
        avoid the pickling of the logger
        """
        d = self.__dict__.copy()
        # disable multiprocessing when saving
        d['n_jobs'] = 1
        d['progress_bar'] = False
        if 'logger' in d:
            d['logger'] = d['logger'].name
        return d

    def __setstate__(self, d):
        """To override the default pickling behavior and
        avoid the pickling of the logger"""
        if 'logger' in d:
            d['logger'] = logging.getLogger(d['logger'])
        self.__dict__.update(d)

    def fit(self, embedding):
        """
        Computes the Lexicons for the specific embedding.
        Parameters
        -------
        embedding : Embedding Object,
            A Melusine Embedding object.

        """

        if self.extend_seed_word_list:
            self.seed_dict, self.seed_list = self.compute_seeds_from_root(embedding, self.base_seed_words)

        self.seed_list = [token for token in self.seed_list if token in embedding.embedding.vocab.keys()]

        if not self.seed_list:
            raise ValueError('None of the seed words are in the vocabulary associated with the Embedding')

        self.lexicon_dict = self.compute_lexicon(embedding, self.seed_list)

        if self.normalize_lexicon_scores:
            self.normalize_lexicon()

    @staticmethod
    def compute_seeds_from_root(embedding, base_seed_words):
        """
        Uses the seedwords list provided by the User and treats them as prefixes to find the effective tokens that will be used to compute the Lexicons.

        Parameters
        --------
        embedding : Embedding Object,
            A Melusine Embedding object.
        base_seed_words :list,
            Seedwords list containing the seedwords for computing the Lexicons given by the User.

        Returns
        -------
        (seed_dict, seed_list) : (dict, list),
            Tuple contraining a dict with key:prefixe value : list of seedwords, and a list containing all the seedwords found with the given prefixes

        """
        words = list(embedding.embedding.vocab.keys())
        seed_dict = dict()
        seed_list = []

        for seed in base_seed_words:
            extended_seed_words = [token for token in words if token.startswith(seed)]
            seed_dict[seed] = extended_seed_words
            seed_list.extend(extended_seed_words)

        return seed_dict, seed_list

    @staticmethod
    def compute_lexicon(embedding, seed_list):
        """
        Computes the Lexicons for the given embedding. Computes the cosine similarity between all the tokens in seed_list and the embedding's vocabulary.

        Parameters
        --------
        embedding : Embedding Object,
            A Melusine Embedding object on which the cosine similarity between the words will be computed.
        seed_list : list,
            A list containing the seedwords on which the cosine similarity will be computed.

        Returns
        -------
        lexicon_dict : dict,
            A dict representing the lexicon. The keys will be all the tokens in seed_list, the values will be a dict which keys will be all the tokens of the embedding's vocabulary, and the values their cosine similarity with the seed.

        """
        words = list(embedding.embedding.vocab.keys())
        lexicon_dict = {}

        for seed in seed_list:
            lexicon_dict[seed] = {}
            for word in words:
                lexicon_dict[seed][word] = embedding.embedding.similarity(seed, word)

        return lexicon_dict

    def predict(self, X, return_column='score'):
        """
        Given the objet has already been fitted, will add a new column "score"
        (or the column name specified as argument) to the Pandas Dataset containing the polarity scores of the
        documents towards the list of seeds provided.
        Parameters
        ----------
        X : DataFrame
            Input emails DataFrame
        return_column : str
            Name of the new column added to the DataFrame containing the semantic score

        """
        X[return_column] = apply_by_multiprocessing(X, self.rate_email, workers=self.n_jobs,
                                                    progress_bar=self.progress_bar)

        return X

    def normalize_lexicon(self) :
        """
        Normalizes the Lexicon scores (centered around 0, with variance 1).
        """
        lexicon_dict = self.lexicon_dict

        normalized_lexicon=dict()
        for seed in lexicon_dict.keys() :
            mean=np.mean(list(lexicon_dict[seed].values()))
            sd=np.std(list(lexicon_dict[seed].values()))
            lex_norm={k:(v-mean)/sd for k,v in lexicon_dict[seed].items()}
            normalized_lexicon[seed]=lex_norm

        self.normalized_lexicon_dict = normalized_lexicon

    def rate_email(self, row):
        """
        Given the object has been fitted, will compute the polarity score towards the seedwords for a specific e-mail.

        Parameters
        ----------
        row : row,
            A Pandas Dataframe containing a tokenized document.
        """

        # TODO make the aggregation function as an argument
        tokens_column = self.tokens_column
        seed_list = self.seed_list

        if self.normalize_lexicon_scores:
            lexicon_dict = self.normalized_lexicon_dict
        else:
            lexicon_dict = self.lexicon_dict

        effective_tokens_list = [token for token in row[tokens_column] if token in lexicon_dict[seed_list[0]]]

        if effective_tokens_list:

            token_score_list = [
                self.aggregation_function_seed_wise(
                    [lexicon_dict[seed][token] for seed in seed_list]
                )
                for token in effective_tokens_list
            ]

            return self.aggregation_function_email_wise(token_score_list)
        else :
            return(np.nan)



def extract_last_body(row):
    """Extracts the body from the last message of the conversation.
    The conversation is structured as a dictionary.

    To be used with methods such as: `apply(func, axis=1)` or
    `apply_by_multiprocessing(func, axis=1, **kwargs)`.

    Parameters
    ----------
    message_dict : dict

    Returns
    -------
    str

    """
    last_message_dict = row["structured_body"][0]
    last_body = extract_body(last_message_dict)

    return last_body


def extract_body(message_dict):
    """Extracts the body from a message dictionary.

    Parameters
    ----------
    message_dict : dict

    Returns
    -------
    str

    """
    tagged_parts_list = message_dict["structured_text"]["text"]
    body = ""
    for part_tag_dict in tagged_parts_list:
        part = part_tag_dict["part"]
        tag = part_tag_dict["tags"]
        if tag == 'BODY':
            body += part + " "
        elif tag == 'GREETINGS':
            break

    return body


def extract_header(message_dict):
    """Extracts the header from a message dictionary.

    Parameters
    ----------
    message_dict : dict

    Returns
    -------
    str

    """
    header = message_dict["structured_text"]["header"]

    return header



import pandas as pd
import numpy as np
from melusine.prepare_email.body_header_extraction import extract_last_body
from melusine.prepare_email.body_header_extraction import extract_body
from melusine.prepare_email.body_header_extraction import extract_header


structured_body = [
    {'meta': {'date': None,
              'from': None,
              'to': None},
     'structured_text': {'header': 'demande document',
                         'text': [{'part': 'Bonjour. ', 'tags': 'HELLO'},
                                  {'part': "Je vous remercie pour le document",
                                   'tags': 'BODY'},
                                  {'part': 'Cordialement,', 'tags': 'GREETINGS'},
                                  {'part': 'Mr Unknown', 'tags': 'BODY'}
                                  ]
                         }
     },
    {'meta': {'date': ' mar. 22 mai 2018 à 10:20',
              'from': '  <destinataire@gmail.fr> ',
              'to': None},
     'structured_text': {'header': 'demande document',
                         'text': [{'part': 'Bonjour. ', 'tags': 'HELLO'},
                                  {'part': "Merci de bien vouloir prendre connaissance du document ci-joint",
                                              'tags': 'BODY'},
                                  {'part': 'Cordialement,', 'tags': 'GREETINGS'},
                                  {'part': 'Votre mutuelle', 'tags': 'BODY'},
                                  {'part': 'La visualisation des fichiers PDF nécessite Adobe Reader.',
                                   'tags': 'FOOTER'}
                                  ]
                         }
     }
    ]


def test_extract_last_body():
    input_df = pd.DataFrame({
        'structured_body': [structured_body]
    })

    output_df = pd.Series(["Je vous remercie pour le document "])

    result = input_df.apply(extract_last_body, axis=1)
    pd.testing.assert_series_equal(result, output_df)


message_dict = {'meta': {'date': ' mar. 22 mai 2018 à 10:20',
                         'from': '  <destinataire@gmail.fr> ',
                         'to': None},
                'structured_text': {'header': 'demande document',
                                    'text': [{'part': 'Bonjour. ',
                                              'tags': 'HELLO'},
                                             {'part': "Merci de bien vouloir prendre connaissance du document ci-joint",
                                              'tags': 'BODY'},
                                             {'part': 'Cordialement,',
                                              'tags': 'GREETINGS'},
                                             {'part': 'Votre mutuelle',
                                              'tags': 'BODY'},
                                             {'part': 'La visualisation des fichiers PDF nécessite Adobe Reader.',
                                              'tags': 'FOOTER'}
                                            ]
                                    }
                }


def test_extract_body():
    input_dict = message_dict

    output = "Merci de bien vouloir prendre connaissance du document ci-joint "

    result = extract_body(input_dict)
    np.testing.assert_string_equal(result, output)


def test_extract_header():
    input_dict = message_dict

    output = 'demande document'

    result = extract_header(input_dict)
    np.testing.assert_string_equal(result, output)
